﻿using Eingabeverwaltung.Metamodell;
using System.Collections.Generic;
using System;
using Ueberpruefung;

namespace Testverwaltung
{
	/// <summary>
	/// This class includes a few manual tests.
	/// </summary>
	public class ManualTests
	{
		private readonly Transformation tr;
		public ManualTests(Container container)
		{
			tr = container.Proof.proof as Transformation;
			doAllTests();
		}

		public void doAllTests()
		{
			testCheckHelper_Substitute();
			testChechHelper_FindSubstitution();
			testCheckHelper_transform();
			testTree_equals();
		}

		public void testCheckHelper_Substitute()
		{
			Tree substTree = CheckHelper.substitute(tr.transformationSteps[0].ruleStartTree(), tr.transformationSteps[0].substitutions);
			Console.WriteLine("Test CheckHelper.substitute\n" +
							  "Input: Tree " + tr.transformationSteps[0].ruleStartTree().ToString() + "\n" +
							  "       Substitution " + List_Substitution_ToString(tr.transformationSteps[0].substitutions) + "\n" +
							  "Result: " + substTree.ToString() + "\n");
		}

		public void testChechHelper_FindSubstitution()
		{
			Tree forTree = tr.transformationSteps[0].ruleStartTree();
			Tree toTree = tr.transformationSteps[0].termPart;
			List<Substitution> subs = CheckHelper.findSubstitution(forTree, toTree);
			Console.WriteLine("Test CheckHelper.findSubstitution\n" +
							  "Input: forTree " + forTree + "\n" +
							  "       toTree  " + toTree + "\n" +
							  "Result: " + List_Substitution_ToString(subs) + "\n");
		}

		public void testCheckHelper_transform()
		{
			Tree newSubtree = CheckHelper.substitute(tr.transformationSteps[0].ruleEndTree()
													, tr.transformationSteps[0].substitutions
													);
			Tree transformedtree = CheckHelper.transform(tr.startTree
														, tr.transformationSteps[0].termPart
														, newSubtree
														);
			Console.WriteLine("Test CheckHelper.transform\n" +
							  "Input: oldTree " + tr.startTree + "\n" +
							  "       subtree " + tr.transformationSteps[0].termPart + "\n" +
							  "       newSubTree " + newSubtree + "\n" +
							  "Result: " + transformedtree.ToString() + "\n");
		}

		public void testTree_equals()
		{
			Tree tree = CheckHelper.substitute(tr.transformationSteps[0].ruleStartTree(), tr.transformationSteps[0].substitutions);
			Tree this_ = tr.transformationSteps[0].termPart;
			Console.WriteLine("Test Tree.equals\n" +
							  "Input: this " + this_ + "\n" +
							  "       Tree  " + tree + "\n" +
							  "Result: " + this_.Equals(tree) + "\n");
		}

		private string List_Substitution_ToString(List<Substitution> subs)
		{
			string result = "[ ";
			foreach (Substitution sub in subs)
				result += sub.tree.ToString() + " / " + sub.variable.name + ",";
			result = result.Remove(result.Length - 1);
			result += "]";
			return result;
		}
	}
}
